library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity EDXSPL2_TOP is
	Port ( CLK  : in  std_logic; -- SYSTEM CLOCK	       
	       PSW0 : in std_logic;  -- RESET
	       PSW1 : in std_logic;  -- STOP / START
			 PSW3 : in std_logic;  -- SYSTEM RESET

			 SG   : out std_logic_vector( 6 downto 0 );
			 SA   : out std_logic_vector( 3 downto 0 ) );
end EDXSPL2_TOP;

architecture RTL of EDXSPL2_TOP is

component STATE_MACHINE is
    Port ( CLK           : in std_logic;
           RST           : in std_logic;
			  STOP_SW       : in std_logic;
			  STATE_OUT     : out std_logic );
end component;

component CHATTERING is
  port ( CLK   : in   std_logic;
  			CE    : in   std_logic;
  		   RST   : in   std_logic;
         A_IN  : in   std_logic;
         B_IN  : in   std_logic;
         A_OUT : out  std_logic;
         B_OUT : out  std_logic );
end component;

component CNT60 is
    Port ( CLK           : in std_logic;
           RST           : in std_logic;
			  CE            : in std_logic;

			  RESET_SW      : in std_logic;

			  STATE         : in std_logic;

           SEC6          : out std_logic_vector(3 downto 0);
           SEC10         : out std_logic_vector(3 downto 0));
end component;

component CLKDIV is
  generic ( WIDTH : integer := 25;  
 	             N : integer := 18432000 );

  port    (   CLK   : in  std_logic;
  				  RST   : in  std_logic;
              SEC   : out std_logic );
end component;

component SEG_CTL is
	port( CLK, RST                       : in  std_logic;
			DATA_A, DATA_B, DATA_C, DATA_D : in  std_logic_vector( 3 downto 0 );
			DATA_OUT                       : out std_logic_vector( 6 downto 0 );
			SA                             : out std_logic_vector( 3 downto 0 ) );
end component;

	signal RST    : std_logic;
	signal SEC_CE : std_logic;
	signal SW_CE  : std_logic;
	signal SEC6   : std_logic_vector( 3 downto 0 );
	signal SEC10  : std_logic_vector( 3 downto 0 );

	signal STOP_SW    : std_logic;
	signal RESET_SW   : std_logic;
	signal STOP_START : std_logic;

begin

	RST    <= not PSW3; -- SYSTEM RESET

	U0 : SEG_CTL port map 
		 ( CLK      => CLK,
			RST      => RST,            

			DATA_A   => SEC10(3 downto 0), -- RIGHT
			DATA_B   => SEC6 (3 downto 0),
			DATA_C   => "0000",
			DATA_D   => "0000",            -- LEFT

			DATA_OUT => SG,                     
			SA       => SA  );

	-- SEC_CE : 1 Hz
	U1 : CLKDIV generic map ( 25, 18432000 )
	               port map ( CLK, RST, SEC_CE );
	-- SW_CE :  100Hz
	U2 : CLKDIV generic map(  18, 184320 )
	               port map ( CLK, RST, SW_CE );

	U3 : CHATTERING port map 
	( CLK   => CLK,
     CE    => SW_CE,
     RST   => RST,
     A_IN  => PSW0,
     B_IN  => PSW1,
     A_OUT => STOP_SW,
     B_OUT => RESET_SW );

   U4 : CNT60 port map
	  ( CLK => CLK,
		 RST => RST,
		 CE  => SEC_CE,
 	    RESET_SW => RESET_SW,
		 STATE    => STOP_START,
		 SEC6     => SEC6,
		 SEC10    => SEC10 );

   U5 : STATE_MACHINE port map
	( CLK       => CLK,
     RST       => RST,
	  STOP_SW   => STOP_SW,
	  STATE_OUT => STOP_START );

end RTL;
