library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity SEG_CTL is
	port( CLK, RST                       : in  std_logic;
			DATA_A, DATA_B, DATA_C, DATA_D : in  std_logic_vector( 3 downto 0 );
			DATA_OUT                       : out std_logic_vector( 6 downto 0 );
			SA                             : out std_logic_vector( 3 downto 0 ) );
end SEG_CTL;

architecture RTL of SEG_CTL  is

component SEG_DECODER is
	port ( D_IN  : in  std_logic_vector( 3 downto 0 );
	       D_OUT : out std_logic_vector( 6 downto 0 ) );
end component;

component SEG_MUX
	port( DATA_A, DATA_B, DATA_C, DATA_D : in  std_logic_vector( 6 downto 0 );
			SEL                            : in  std_logic_vector( 1 downto 0 );
			DATA_OUT                       : out std_logic_vector( 6 downto 0 );
			SA                             : out std_logic_vector( 3 downto 0 ) );
end component;

component SEG_CNT
	port ( CLK : in  std_logic; 
	       RST : in  std_logic;
			 STB : out std_logic_vector( 1 downto 0 ) );
end component;

signal DEC_A : std_logic_vector( 6 downto 0 ); 
signal DEC_B : std_logic_vector( 6 downto 0 ); 
signal DEC_C : std_logic_vector( 6 downto 0 ); 
signal DEC_D : std_logic_vector( 6 downto 0 );

signal SEL   : std_logic_vector( 1 downto 0 );

begin
	U0 : SEG_DECODER port map ( DATA_A, DEC_A );
	U1 : SEG_DECODER port map ( DATA_B, DEC_B );
	U2 : SEG_DECODER port map ( DATA_C, DEC_C );
	U3 : SEG_DECODER port map ( DATA_D, DEC_D );

	U6 : SEG_CNT port map ( CLK, RST, SEL );
	U7 : SEG_MUX port map ( DEC_A, DEC_B, DEC_C, DEC_D, SEL, DATA_OUT, SA );
end RTL;

