library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity STATE_MACHINE is
    Port ( CLK           : in std_logic;
           RST           : in std_logic;
			  STOP_SW       : in std_logic;
			  STATE_OUT     : out std_logic );
end STATE_MACHINE;

architecture RTL of STATE_MACHINE is

signal STATE : std_logic_vector(1 downto 0);

constant S0 : std_logic_vector(1 downto 0) := "00";
constant S1 : std_logic_vector(1 downto 0) := "01";
constant S2 : std_logic_vector(1 downto 0) := "10";
constant S3 : std_logic_vector(1 downto 0) := "11";

begin

process( CLK, RST ) begin
	if ( RST ='1') then
		STATE <= S0;
		STATE_OUT <= '0';
	elsif( CLK'event and CLK='1') then
		case STATE is
			when S0 => -- START
				if( STOP_SW='1') then
					STATE_OUT <= '1';
					STATE <= S1;
				else						
					STATE <= S0;
				end if;
			when S1 =>
				if( STOP_SW='0') then
					STATE <= S2;
				else
					STATE <= S1;
				end if;
			when S2 => -- STOP
				if( STOP_SW='1') then
					STATE_OUT <= '0';
					STATE <= S3;
				else						
					STATE <= S2;
				end if;
			when S3 =>
				if( STOP_SW='0') then
					STATE <= S0;
				else
					STATE <= S3;
				end if;
			when others => STATE <= S0;
		end case;
	end if;
end process;

end RTL;
